@@ -1,14 +1,17 @@
 # SPDX-License-Identifier: GPL-3.0-only
 from __future__ import annotations
 
+import io
+import json
 import logging
 from pathlib import Path
 
-from gi.repository import Gio, GLib, GObject
+from gi.repository import Gio, GLib, GObject, Gtk
 from pykeepass import PyKeePass
 
 import gsecrets.config_manager as config
 from gsecrets.safe_element import SafeEntry, SafeGroup
+from gsecrets.utils import compare_passwords
 
 QUARK = GLib.quark_from_string("secrets")
 
@@ -16,8 +19,7 @@ QUARK = GLib.quark_from_string("secrets")
 class DatabaseManager(GObject.Object):
     # pylint: disable=too-many-public-methods
     # pylint: disable=too-many-instance-attributes
-
-    """Implements database functionality that is independent of the UI
+    """Implements database functionality that is independent of the UI.
 
     Useful attributes:
      .path: str containing the filepath of the database
@@ -43,8 +45,7 @@ class DatabaseManager(GObject.Object):
     # Only used for setting the credentials to their actual values in case of
     # errors.
     old_password: str = ""
-    old_keyfile: str = ""
-    old_keyfile_hash: str = ""
+    composition_key: bytes | None = None
 
     trash_bin: SafeGroup | None = None
     root: SafeGroup | None = None
@@ -55,7 +56,7 @@ class DatabaseManager(GObject.Object):
     # To be emitted when the elements list model needs to be re sorted.
     sorting_changed = GObject.Signal(arg_types=(bool,))
 
-    def __init__(self, database_path: str) -> None:
+    def __init__(self, key_providers: list, database_path: str) -> None:
         """Initialize the database handling logic.
 
         :param str database_path: The database path
@@ -65,15 +66,14 @@ class DatabaseManager(GObject.Object):
         self.entries = Gio.ListStore.new(SafeEntry)
         self.groups = Gio.ListStore.new(SafeGroup)
 
+        self._key_providers = key_providers
         self._path = database_path
         self.db: PyKeePass = None
-        self.keyfile_hash: str = ""
 
     def unlock_async(
         self,
         password: str,
-        keyfile: str = "",
-        keyfile_hash: str = "",
+        composition_key: bytes | None = None,
         callback: Gio.AsyncReadyCallback = None,
     ) -> None:
         """Unlocks and opens a safe.
@@ -82,8 +82,7 @@ class DatabaseManager(GObject.Object):
         be opened, an exception is raised.
 
         :param str password: password to use or an empty string
-        :param str keyfile: keyfile path to use or an empty string
-        :param str keyfile_hash: keyfile_hash to set
+        :param str composition_key: composition_key as bytes
         :param GAsyncReadyCallback: callback run after the unlock operation ends
         """
         self._opened = False
@@ -92,18 +91,24 @@ class DatabaseManager(GObject.Object):
             if Path(self._path).suffix == ".kdb":
                 # NOTE kdb is a an older format for Keepass databases.
                 err = GLib.Error.new_literal(
-                    QUARK, "The kdb Format is not Supported", 0
+                    QUARK,
+                    "The kdb Format is not Supported",
+                    0,
                 )
                 task.return_error(err)
                 return
 
             try:
-                db = PyKeePass(self.path, password, keyfile)
+                db = PyKeePass(
+                    self.path,
+                    password,
+                    io.BytesIO(composition_key) if composition_key else None,
+                )
             except Exception as err:  # pylint: disable=broad-except
                 err = GLib.Error.new_literal(QUARK, str(err), 1)
                 task.return_error(err)
             else:
-                self.keyfile_hash = keyfile_hash
+                self.composition_key = composition_key
                 self._update_file_monitor()
                 task.return_value(db)
 
@@ -111,27 +116,28 @@ class DatabaseManager(GObject.Object):
         task.run_in_thread(unlock_task)
 
     def unlock_finish(self, result):
-        try:
-            _success, db = result.propagate_value()
-        except GLib.Error as err:
-            raise err
-        else:
-            self.db = db
-            self._opened = True
-            logging.debug("Opening of safe %s was successful", self.path)
+        """Finish unlocking.
 
-            if not self._elements_loaded:
-                self.entries.splice(0, 0, [SafeEntry(self, e) for e in db.entries])
-                self.groups.splice(0, 0, [SafeGroup(self, g) for g in db.groups])
+        Can raise errors.
+        """
+        _success, db = result.propagate_value()
 
-                self._elements_loaded = True
+        self.db = db
+        self._opened = True
+        logging.debug("Opening of safe %s was successful", self.path)
+
+        if not self._elements_loaded:
+            self.entries.splice(0, 0, [SafeEntry(self, e) for e in db.entries])
+            self.groups.splice(0, 0, [SafeGroup(self, g) for g in db.groups])
+
+            self._elements_loaded = True
 
     #
     # Database Modifications
     #
 
     def save_async(self, callback: Gio.AsyncReadyCallback) -> None:
-        """Write all changes to database
+        """Write all changes to database.
 
         This consists in two parts, we first save it to a byte stream in memory
         and then we write the contents of the stream into the database using
@@ -139,7 +145,8 @@ class DatabaseManager(GObject.Object):
         whereas GFile operations are.
 
         Note that certain operations in pykeepass can fail at the middle of the
-        operation, nuking the database in the process."""
+        operation, nuking the database in the process.
+        """
         task = Gio.Task.new(self, None, callback)
         task.run_in_thread(self._save_task)
 
@@ -167,24 +174,26 @@ class DatabaseManager(GObject.Object):
             task.return_boolean(True)
 
     def save_finish(self, result: Gio.AsyncResult) -> bool:
-        """Finishes save_async, returns whether the safe was saved.
-        Can raise GLib.Error."""
-        self.save_running = False
-        try:
-            is_saved = result.propagate_boolean()
-        except GLib.Error as err:
-            raise err
-        else:
-            self.is_dirty = False
-            if is_saved:
-                logging.debug("Database %s saved successfully", self.path)
+        """Finish save_async.
 
-            return is_saved
+        Returns whether the safe was saved. Can raise GLib.Error.
+        """
+        self.save_running = False
+        is_saved = result.propagate_boolean()
+
+        self.is_dirty = False
+        if is_saved:
+            logging.debug("Database %s saved successfully", self.path)
+
+        return is_saved
 
     def set_credentials_async(
-        self, password, keyfile="", keyfile_hash="", callback=None
-    ):
-        """Sets credentials for safe
+        self,
+        password: str,
+        composition_key: bytes | None = None,
+        callback: Gio.AsyncReadyCallback = None,
+    ) -> None:
+        """Set credentials for safe.
 
         It does almost the same as save_async, with the difference that
         correctly handles errors, it won't leave the database in a state where
@@ -195,11 +204,13 @@ class DatabaseManager(GObject.Object):
             self.old_password = self.password
             self.password = password
 
-            self.old_keyfile = self.keyfile
-            self.keyfile = keyfile
+            self.composition_key = composition_key
+            if composition_key:
+                self.db.keyfile = io.BytesIO(composition_key)
+            else:
+                self.db.keyfile = None
 
-            self.old_keyfile_hash = self.keyfile_hash
-            self.keyfile_hash = keyfile_hash
+            self.is_dirty = True
 
             self._save_task(task, obj, data, cancellable)
 
@@ -210,60 +221,53 @@ class DatabaseManager(GObject.Object):
         self.save_running = False
         try:
             is_saved = result.propagate_boolean()
-        except GLib.Error as err:
+        except GLib.Error:
             self.password = self.old_password
-            self.keyfile = self.old_keyfile
-            self.keyfile_hash = self.old_keyfile_hash
 
-            raise err
-        else:
-            self.is_dirty = False
-            if is_saved:
-                logging.debug("Credentials changed successfully")
+            raise
 
-            return is_saved
+        self.is_dirty = False
+        if is_saved:
+            logging.debug("Credentials changed successfully")
+
+        return is_saved
 
     def add_to_history(self) -> None:
-        # Add database uri to history.
         uri = Gio.File.new_for_path(self._path).get_uri()
-        uri_list = config.get_last_opened_list()
 
-        if uri in uri_list:
-            uri_list.sort(key=uri.__eq__)
-        else:
-            uri_list.append(uri)
-
-        config.set_last_opened_list(uri_list)
+        recents = Gtk.RecentManager.get_default()
+        recents.add_item(uri)
 
         # Set last opened database.
         config.set_last_opened_database(uri)
 
-        # Add keyfile path to history.
         if not config.get_remember_composite_key():
             return
 
-        def filter_func(pair):
-            return pair[0] != uri
+        keyprovider_pairs = config.get_last_used_key_provider()
 
-        keyfile_pairs = config.get_last_used_composite_key()
-        new_pairs = list(filter(filter_func, keyfile_pairs))
+        data = {}
+        for key_provider in self._key_providers:
+            if key_provider.available and key_provider.key:
+                provider_config = key_provider.config()
+                data[type(key_provider).__name__] = provider_config
 
-        if self.keyfile:
-            new_pairs.append([uri, self.keyfile])
-
-        config.set_last_used_composite_key(new_pairs)
+        keyprovider_pairs[uri] = json.dumps(data)
+        config.set_last_used_key_provider(keyprovider_pairs)
 
     def _update_file_monitor(self):
-        """Updates the modified time and size of the database file, this is a
-        blocking operation."""
+        """Update the modified time and size of the database.
+
+        This is a blocking operation.
+        """
         gfile = Gio.File.new_for_path(self._path)
         attributes = (
             f"{Gio.FILE_ATTRIBUTE_STANDARD_SIZE},{Gio.FILE_ATTRIBUTE_TIME_MODIFIED}"
         )
         try:
             info = gfile.query_info(attributes, Gio.FileQueryInfoFlags.NONE, None)
-        except GLib.Error as err:
-            logging.error("Could not read file size: %s", err.message)
+        except GLib.Error:
+            logging.exception("Could not read file size")
         else:
             self.file_size = info.get_size()
             self.file_mtime = info.get_modification_date_time().to_unix()
@@ -292,36 +296,23 @@ class DatabaseManager(GObject.Object):
                 task.return_boolean(False)
 
     def check_file_changes_finish(self, result: Gio.AsyncResult) -> bool:
-        """Finishes check_file_changes_async.
-        Returns whether the file was changed, can raise GLib.Error."""
-        try:
-            changed = result.propagate_boolean()
-        except GLib.Error as err:
-            raise err
-        else:
-            return changed
+        """Finish check_file_changes_async.
+
+        Returns whether the file was changed, can raise GLib.Error.
+        """
+        return result.propagate_boolean()
 
     @property
     def password(self) -> str:
-        """Get the current password or '' if not set"""
+        """Get the current password or '' if not set."""
         return self.db.password or ""
 
     @password.setter
     def password(self, new_password: str | None) -> None:
-        """Set database password (None if a keyfile is used)"""
+        """Set database password (None if a old_composition key is used)."""
         self.db.password = new_password
         self.is_dirty = True
 
-    @property
-    def keyfile(self) -> str:
-        """Get the current keyfile or None if it is not set."""
-        return self.db.keyfile
-
-    @keyfile.setter
-    def keyfile(self, new_keyfile: str | None) -> None:
-        self.db.keyfile = new_keyfile
-        self.is_dirty = True
-
     #
     # Read Database
     #
@@ -329,7 +320,11 @@ class DatabaseManager(GObject.Object):
     # Check if entry with title in group exists
     def check_entry_in_group_exists(self, title, group):
         entry = self.db.find_entries(
-            title=title, group=group, recursive=False, history=False, first=True
+            title=title,
+            group=group,
+            recursive=False,
+            history=False,
+            first=True,
         )
         if entry is None:
             return False
@@ -344,17 +339,15 @@ class DatabaseManager(GObject.Object):
         self.password_try = password
 
     def compare_passwords(self, password2: str) -> bool:
-        """Compare the first password entered by the user with the second one
+        """Compare the first password entered by the user with the second one.
 
         It also does not allow empty passwords.
         :returns: True if passwords match and are non-empty.
         """
-        if password2 and self.password_try == password2:
-            return True
-        return False
+        return compare_passwords(self.password_try, password2)
 
     def parent_checker(self, current_group, moved_group):
-        """Returns True if moved_group is an ancestor of current_group"""
+        """Return True if moved_group is an ancestor of current_group."""
         # recursively invoke ourself until we reach the root group
         if current_group.is_root_group:
             return False
@@ -364,7 +357,7 @@ class DatabaseManager(GObject.Object):
 
     @property
     def version(self):
-        """returns the database version"""
+        """Returns the database version."""
         return self.db.version
 
     @property
@@ -374,3 +367,7 @@ class DatabaseManager(GObject.Object):
     @property
     def path(self) -> str:
         return self._path
+
+    def get_salt(self) -> bytes:
+        db = PyKeePass(self.path, None, None, decrypt=False)
+        return db.database_salt
