@@ -4,11 +4,11 @@ from __future__ import annotations
 import logging
 import os
 import typing
-from enum import IntEnum
 from gettext import gettext as _
 from gettext import ngettext
+from pathlib import Path
 
-from gi.repository import Adw, Gdk, Gio, GLib, GObject, Gtk
+from gi.repository import Adw, Gio, GLib, GObject, Gtk
 
 import gsecrets.config_manager
 from gsecrets import const
@@ -16,16 +16,14 @@ from gsecrets.entry_page import EntryPage
 from gsecrets.entry_row import EntryRow
 from gsecrets.group_page import GroupPage
 from gsecrets.group_row import GroupRow
-from gsecrets.pathbar import Pathbar
 from gsecrets.safe_element import SafeElement, SafeEntry, SafeGroup
+from gsecrets.selection_manager import SelectionManager
 from gsecrets.unlocked_headerbar import UnlockedHeaderBar
+from gsecrets.widgets.browsing_panel import BrowsingPanel
 from gsecrets.widgets.database_settings_dialog import DatabaseSettingsDialog
 from gsecrets.widgets.properties_dialog import PropertiesDialog
 from gsecrets.widgets.references_dialog import ReferencesDialog
 from gsecrets.widgets.saving_conflict_dialog import SavingConflictDialog
-from gsecrets.widgets.search import Search
-from gsecrets.widgets.selection_mode_headerbar import SelectionModeHeaderbar
-from gsecrets.widgets.unlocked_database_page import UnlockedDatabasePage
 
 if typing.TYPE_CHECKING:
     from gsecrets.database_manager import DatabaseManager
@@ -38,8 +36,25 @@ class UndoData:
         self.toast = toast
 
 
+class AttributeUndoData:
+    # pylint: disable=too-many-arguments
+    def __init__(
+        self,
+        toast: Adw.Toast,
+        entry: SafeEntry,
+        key: str,
+        value: str,
+        protected: bool = False,
+    ):
+        self.toast = toast
+        self.key = key
+        self.value = value
+        self.entry = entry
+        self.protected = protected
+
+
 @Gtk.Template(resource_path="/org/gnome/World/Secrets/gtk/unlocked_database.ui")
-class UnlockedDatabase(Gtk.Box):
+class UnlockedDatabase(Adw.BreakpointBin):
     # pylint: disable=too-many-instance-attributes
     # pylint: disable=too-many-public-methods
     __gtype_name__ = "UnlockedDatabase"
@@ -47,33 +62,34 @@ class UnlockedDatabase(Gtk.Box):
     # Connection handlers
     db_locked_handler: int | None = None
     # Used only to delete the clipboard if the window is not active.
-    clipboard_last_content: Gdk.ContentProvider | None = None
-    clipboard_timer_handler: int | None = None
     on_is_active_handler: int | None = None
     _current_element: SafeElement | None = None
     _lock_timer_handler: int | None = None
     save_loop: int | None = None  # If int, a thread periodically saves the database
     session_handler_id: int | None = None
 
-    action_bar = Gtk.Template.Child()
-    _edit_page_bin = Gtk.Template.Child()
     headerbar_stack = Gtk.Template.Child()
-    _main_view = Gtk.Template.Child()
-    _stack = Gtk.Template.Child()
-    _unlocked_db_deck = Gtk.Template.Child()
-    _unlocked_db_stack = Gtk.Template.Child()
-    search_bar = Gtk.Template.Child()
-    search_entry = Gtk.Template.Child()
+    _split_view = Gtk.Template.Child()
+    toolbar_view = Gtk.Template.Child()
+    _select_entry_status_page = Gtk.Template.Child()
+    _empty_page = Gtk.Template.Child()
+
+    _search_active: bool = False
+    _search_bar = Gtk.Template.Child()
+    _search_entry = Gtk.Template.Child()
+
+    _selection_mode: bool = False
+    _selection_mode_headerbar = Gtk.Template.Child()
+    _selection_mode_action_bar = Gtk.Template.Child()
+    _delete_selection_button = Gtk.Template.Child()
+    _cut_selection_button = Gtk.Template.Child()
+    _paste_selection_button = Gtk.Template.Child()
+    _selection_mode_title = Gtk.Template.Child()
+    _cut_paste_button_stack = Gtk.Template.Child()
+    _clear_selection_button = Gtk.Template.Child()
 
-    selection_mode = GObject.Property(type=bool, default=False)
     undo_data: UndoData | None = None
-
-    class Mode(IntEnum):
-        ENTRY = 0
-        GROUP = 1
-        GROUP_EDIT = 2
-        SEARCH = 3
-        SELECTION = 4
+    attribute_undo_data: AttributeUndoData | None = None
 
     def __init__(self, window: Window, dbm: DatabaseManager):
         super().__init__()
@@ -83,36 +99,28 @@ class UnlockedDatabase(Gtk.Box):
 
         root_group = SafeGroup.get_root(dbm)
         self.props.current_element = root_group
-        self._mode = self.Mode.GROUP
-
-        # Actionbar has to be setup before edit entry & group headerbars.
-        mobile_pathbar = Pathbar(self)
-        self.pathbar = Pathbar(self)
-        self.action_bar.pack_start(mobile_pathbar)
 
         # Headerbars
-        self.selection_mode_headerbar = SelectionModeHeaderbar(self)
         self.headerbar = UnlockedHeaderBar(self)
 
-        self.headerbar_stack.add_child(self.selection_mode_headerbar)
         self.headerbar_stack.add_child(self.headerbar)
+        self.headerbar_stack.props.visible_child = self.headerbar
 
-        # self.search has to be loaded after the search headerbar.
-        self._search_active = False
-        self.search: Search = Search(self)
-        self._unlocked_db_stack.add_named(self.search, "search")
-        self.search.initialize()
+        self._select_entry_status_page.props.icon_name = f"{const.APP_ID}-symbolic"
+
+        self.active_element: SafeElement | None = None
 
         # Browser Mode
-        self.show_browser_page(self.current_element)  # type: ignore
+        self.browsing_panel = BrowsingPanel(self)
+        self.toolbar_view.props.content = self.browsing_panel
 
         self.setup()
 
         self.clipboard = self.get_clipboard()
 
-        self.connect("notify::selection-mode", self._on_selection_mode_changed)
         self.db_locked_handler = self.database_manager.connect(
-            "notify::locked", self._on_database_lock_changed
+            "notify::locked",
+            self._on_database_lock_changed,
         )
 
         # Sets the menu's save button sensitive property.
@@ -124,17 +132,27 @@ class UnlockedDatabase(Gtk.Box):
 
         def on_locked(dbm, _spec):
             if dbm.locked:
-                self.search_bar.set_key_capture_widget(None)
+                self._search_bar.set_key_capture_widget(None)
             else:
-                self.search_bar.set_key_capture_widget(window)
+                self._search_bar.set_key_capture_widget(window)
 
         dbm.connect("notify::locked", on_locked)
-        self.search_bar.set_key_capture_widget(window)
+        self._search_bar.set_key_capture_widget(window)
 
-        self.search_bar.connect_entry(self.search_entry)
-        self.bind_property("search-active", self.search_bar, "search-mode-enabled")
+        self._search_entry.connect("search-changed", self._on_search_changed)
+        self.bind_property(
+            "search-active",
+            self._search_bar,
+            "search-mode-enabled",
+            GObject.BindingFlags.SYNC_CREATE | GObject.BindingFlags.BIDIRECTIONAL,
+        )
 
-    def do_dispose(self):
+        self._split_view.connect("notify::show-content", self._on_show_content_notify)
+        self._split_view.connect("notify::collapsed", self._on_collapsed_notify)
+
+        self._selection_manager = SelectionManager(self)
+
+    def inner_dispose(self):
         logging.debug("Database disposed")
         self.cleanup()
 
@@ -148,71 +166,51 @@ class UnlockedDatabase(Gtk.Box):
 
         app = self.window.props.application
         self.session_handler_id = app.connect(
-            "notify::screensaver-active", self.on_session_lock
+            "notify::screensaver-active",
+            self.on_session_lock,
         )
 
     def listbox_row_factory(self, element: SafeElement) -> Gtk.Widget:
         if element.is_entry:
-            return EntryRow(self, element)
+            row = EntryRow(self)
+            row.props.safe_entry = element
+            return row
 
-        return GroupRow(self, element)
+        row = GroupRow(self)
+        row.props.safe_group = element
+        return row
 
     def show_edit_page(self, element: SafeElement, new: bool = False) -> None:
         self.start_database_lock_timer()
-        self.props.current_element = element
-
-        if self.props.search_active:
-            self.props.search_active = False
+        self.active_element = element
 
         # Sets the accessed time.
         element.touch()
 
         if element.is_group:
-            page = GroupPage(self)
-            self.props.mode = self.Mode.GROUP_EDIT
+            page = GroupPage(self, element)
         else:
-            page = EntryPage(self, new)
-            self.props.mode = self.Mode.ENTRY
+            page = EntryPage(self, element, new)
 
-        self._edit_page_bin.set_child(page)
-        self._unlocked_db_deck.set_visible_child(self._edit_page_bin)
+        page = Adw.NavigationPage.new(page, "Edit")
+        self._split_view.props.content = page
+        self._split_view.props.show_content = True
 
     def show_browser_page(self, group: SafeGroup) -> None:
         self.start_database_lock_timer()
-        page_name = group.uuid.urn
 
-        if page := self._stack.get_child_by_name(page_name):
-            self.props.current_element = page.group
-        else:
-            self.props.current_element = group
-            new_page = UnlockedDatabasePage(self, group)
-            self._stack.add_named(new_page, page_name)
+        self.active_element = group
+        self.props.current_element = group
+        self._split_view.props.show_content = False
+        self.browsing_panel.visit_group(group)
 
-        self._unlocked_db_stack.set_visible_child(self._stack)
-        self._unlocked_db_deck.set_visible_child(self._main_view)
-        if not self.props.selection_mode:
-            self.props.mode = self.Mode.GROUP
+    def _navigate_back(self):
+        if element := self.active_element:
+            parent_group = element.parentgroup
+            self.show_browser_page(parent_group)
+            return
 
-        self._stack.set_visible_child_name(page_name)
-
-    @property
-    def in_edit_page(self) -> bool:
-        """Returns true if the current visible page is either
-        the Group edit page or Entry edit page."""
-
-        boolean: bool = (
-            self._unlocked_db_deck.props.visible_child == self._edit_page_bin
-        )
-        return boolean
-
-    def _on_selection_mode_changed(
-        self, _unlocked_database: UnlockedDatabase, _value: GObject.ParamSpec
-    ) -> None:
-        if self.props.selection_mode:
-            self.props.mode = self.Mode.SELECTION
-            self.headerbar_stack.set_visible_child(self.selection_mode_headerbar)
-        else:
-            self.headerbar_stack.set_visible_child(self.headerbar)
+        self.show_browser_page(self.current_element)
 
     #
     # Group and Entry Management
@@ -220,54 +218,26 @@ class UnlockedDatabase(Gtk.Box):
 
     @GObject.Property(type=SafeElement)
     def current_element(self) -> SafeElement:
+        # FIXME Rename to current_group, check all uses
         return self._current_element
 
     @current_element.setter  # type: ignore
     def current_element(self, element: SafeElement) -> None:
         self._current_element = element
 
-    def get_current_page(self) -> UnlockedDatabasePage:
-        """Returns the page associated with current_element.
-
-        :returns: current page
-        :rtype: Gtk.Widget
-        """
-        element_uuid = self.props.current_element.uuid
-        return self._stack.get_child_by_name(element_uuid.urn)
-
-    def delete_page(self, element):
-        if page := self._stack.get_child_by_name(element.uuid.urn):
-            self._stack.remove(page)
-
     #
     # Events
     #
 
-    def on_list_box_row_activated(self, _widget, list_box_row):
-        self.start_database_lock_timer()
-
-        if self.props.search_active:
-            self.props.search_active = False
-
-        if list_box_row.__gtype_name__ == "GroupRow":
-            safe_group = list_box_row.safe_group
-            self.show_browser_page(safe_group)
-        else:
-            if self.selection_mode:
-                active = list_box_row.selection_checkbox.props.active
-                list_box_row.selection_checkbox.props.active = not active
-                return
-
-            safe_entry = list_box_row.safe_entry
-            self.show_edit_page(safe_entry)
-
     def on_save(
-        self, database_manager: DatabaseManager, result: Gio.AsyncResult
+        self,
+        database_manager: DatabaseManager,
+        result: Gio.AsyncResult,
     ) -> None:
         try:
             is_saved = database_manager.save_finish(result)
-        except GLib.Error as err:
-            logging.error("Could not save Safe: %s", err.message)
+        except GLib.Error:
+            logging.exception("Could not save Safe")
             self.window.send_notification(_("Could not save Safe"))
         else:
             if is_saved:
@@ -276,42 +246,44 @@ class UnlockedDatabase(Gtk.Box):
                 self.window.send_notification(_("No changes made"))
 
     def on_auto_save(
-        self, database_manager: DatabaseManager, result: Gio.AsyncResult
+        self,
+        database_manager: DatabaseManager,
+        result: Gio.AsyncResult,
     ) -> None:
         # TODO Does it make sense to present this info to the user?
         try:
             database_manager.save_finish(result)
-        except GLib.Error as err:
-            logging.error("Could not automatically save Safe %s", err.message)
+        except GLib.Error:
+            logging.exception("Could not automatically save Safe")
 
     def lock_safe(self):
         self.database_manager.props.locked = True
 
     def on_add_entry_action(self) -> None:
-        """CB when the Add Entry menu was clicked"""
+        """CB when the Add Entry menu was clicked."""
         group = self.props.current_element
         new_entry: SafeEntry = group.new_entry()
         self.show_edit_page(new_entry, new=True)
 
     def on_add_group_action(self) -> None:
-        """CB when menu entry Add Group is clicked"""
+        """CB when menu entry Add Group is clicked."""
         group = self.props.current_element
         safe_group = group.new_subgroup()
         self.show_edit_page(safe_group)
 
     def on_element_delete_action(self) -> None:
         """Delete the visible entry from the menu."""
-        element = self.props.current_element
-        parent_group = self.props.current_element.parentgroup
-        if element.trash():
-            self.delete_page(element)
-            elements = []
-        else:
-            elements = [(element, parent_group)]
+        if element := self.active_element:
+            parent_group = element.parentgroup
+            self.browsing_panel.unselect(element)
+            if element.trash():
+                elements = []
+            else:
+                elements = [(element, parent_group)]
 
-        self.deleted_notification(elements)
-
-        self.show_browser_page(parent_group)
+            self._split_view.props.content = self._empty_page
+            self.deleted_notification(elements)
+            self._navigate_back()
 
     def deleted_notification(self, elements):
         n_del = len(elements)
@@ -335,9 +307,8 @@ class UnlockedDatabase(Gtk.Box):
             data.toast.dismiss()
 
         def dismissed_db(toast):
-            if data := self.undo_data:
-                if data.toast == toast:
-                    self.undo_data = None
+            if (data := self.undo_data) and data.toast == toast:
+                self.undo_data = None
 
         toast.connect("dismissed", dismissed_db)
 
@@ -351,11 +322,29 @@ class UnlockedDatabase(Gtk.Box):
 
             self.undo_data = None
 
-    def on_entry_duplicate_action(self):
-        self.props.current_element.duplicate()
-        parent_group = self.props.current_element.parentgroup
+    def attribute_deleted(self, entry, key, value, protected):
+        toast = Adw.Toast.new(_("Attribute deleted"))
+        toast.props.button_label = _("Undo")
+        toast.props.action_name = "win.db.undo_attribute_delete"
 
-        self.show_browser_page(parent_group)
+        if data := self.attribute_undo_data:
+            data.toast.dismiss()
+
+        undo_data = AttributeUndoData(toast, entry, key, value, protected)
+        self.attribute_undo_data = undo_data
+        self.window.toast_overlay.add_toast(toast)
+
+    def undo_attribute_delete(self):
+        if data := self.attribute_undo_data:
+            data.toast.dismiss()
+            data.entry.set_attribute(data.key, data.value, data.protected)
+
+            self.attribute_undo_data = None
+
+    def on_entry_duplicate_action(self):
+        if element := self.active_element:
+            element.duplicate()
+            self._navigate_back()
 
     def send_to_clipboard(
         self,
@@ -368,11 +357,6 @@ class UnlockedDatabase(Gtk.Box):
 
         self.start_database_lock_timer()
 
-        if self.clipboard_timer_handler:
-            GLib.source_remove(self.clipboard_timer_handler)
-            self.clipboard_timer_handler = None
-
-        self.clipboard_last_content = None
         if self.on_is_active_handler:
             self.disconnect(self.on_is_active_handler)
             self.on_is_active_handler = None
@@ -388,13 +372,17 @@ class UnlockedDatabase(Gtk.Box):
             self.window.send_notification(message)
 
         clear_clipboard_time = gsecrets.config_manager.get_clear_clipboard()
+        clipboard_last_content = self.clipboard.get_content()
 
         def callback():
-            GLib.source_remove(self.clipboard_timer_handler)
-            self.clipboard_timer_handler = None
             if window.props.is_active:
-                self.clipboard.set_content(None)
+                if clipboard_last_content == self.clipboard.get_content():
+                    logging.debug("Clipboard cleared")
+                    self.clipboard.set_content(None)
             else:
+                logging.debug(
+                    "Could not clear the clipboard, it will clear on new focus",
+                )
 
                 def on_is_active(window, _pspec):
                     if self.on_is_active_handler:
@@ -402,33 +390,28 @@ class UnlockedDatabase(Gtk.Box):
                         self.on_is_active_handler = None
 
                     # Only clear clipboard if we set ourselves the content.
-                    if self.clipboard_last_content == self.clipboard.get_content():
+                    if clipboard_last_content == self.clipboard.get_content():
                         logging.debug("Clipboard cleared")
                         self.clipboard.set_content(None)
-                    else:
-                        logging.debug("Clipboard not cleared: Content set not by us")
 
-                    self.clipboard_last_content = None
-
-                logging.debug(
-                    "Could not clear the clipboard, it will clear on new focus"
-                )
-                self.clipboard_last_content = self.clipboard.get_content()
                 self.on_is_active_handler = window.connect(
-                    "notify::is-active", on_is_active
+                    "notify::is-active",
+                    on_is_active,
                 )
 
             return GLib.SOURCE_REMOVE
 
-        self.clipboard_timer_handler = GLib.timeout_add_seconds(
-            clear_clipboard_time, callback
-        )
+        GLib.timeout_add_seconds(clear_clipboard_time, callback)
 
     def show_database_settings(self) -> None:
-        DatabaseSettingsDialog(self).present()
+        DatabaseSettingsDialog(self).present(self)
 
     def on_session_lock(self, app: Gtk.Application, _pspec: GObject.ParamSpec) -> None:
-        if app.props.screensaver_active and not self.database_manager.props.locked:
+        if (
+            app.props.screensaver_active
+            and not self.database_manager.props.locked
+            and gsecrets.config_manager.get_lock_on_session_lock()
+        ):
             self.lock_timeout_database()
 
     #
@@ -436,16 +419,18 @@ class UnlockedDatabase(Gtk.Box):
     #
 
     def show_references_dialog(self) -> None:
-        """Show a Group/Entry reference dialog
+        """Show a Group/Entry reference dialog.
 
-        Invoked by the app.entry.references action"""
-        ReferencesDialog(self).present()
+        Invoked by the app.entry.references action
+        """
+        ReferencesDialog(self).present(self)
 
     def show_properties_dialog(self) -> None:
-        """Show a Group/Entry property dialog
+        """Show a Group/Entry property dialog.
 
-        Invoked by the app.element.properties action"""
-        PropertiesDialog(self).present()
+        Invoked by the app.element.properties action
+        """
+        PropertiesDialog(self).present(self)
 
     #
     # Utils
@@ -475,7 +460,7 @@ class UnlockedDatabase(Gtk.Box):
     #
 
     def cleanup(self) -> None:
-        """Stop all ongoing operations:
+        """Stop all ongoing operations.
 
         * stop the save loop
         * cancel all timers
@@ -485,11 +470,6 @@ class UnlockedDatabase(Gtk.Box):
         """
         logging.debug("Cleaning database %s", self.database_manager.path)
 
-        if self.clipboard_timer_handler:
-            GLib.source_remove(self.clipboard_timer_handler)
-            self.clipboard_timer_handler = None
-            self.clipboard.set_content(None)
-
         if self._lock_timer_handler:
             GLib.source_remove(self._lock_timer_handler)
             self._lock_timer_handler = None
@@ -511,14 +491,14 @@ class UnlockedDatabase(Gtk.Box):
         def callback(gfile, result):
             try:
                 gfile.delete_finish(result)
-            except GLib.Error as err:
-                logging.debug("Could not delete temporal file: %s", err.message)
+            except GLib.Error:
+                logging.exception("Could not delete temporal file")
 
-        cache_dir = os.path.join(GLib.get_user_cache_dir(), const.SHORT_NAME, "tmp")
+        cache_dir = Path(GLib.get_user_cache_dir()) / const.SHORT_NAME / "tmp"
         for root, _dirs, files in os.walk(cache_dir):
             for file_name in files:
-                file_path = os.path.join(root, file_name)
-                gfile = Gio.File.new_for_path(file_path)
+                file_path = Path(root) / file_name
+                gfile = Gio.File.new_for_path(str(file_path))
                 gfile.delete_async(GLib.PRIORITY_DEFAULT, None, callback)
 
     def save_database(self) -> None:
@@ -544,13 +524,13 @@ class UnlockedDatabase(Gtk.Box):
     def on_check_file_changes(self, dbm, result, on_save_callback):
         try:
             conflicts = dbm.check_file_changes_finish(result)
-        except GLib.Error as err:
-            logging.error("Could not monitor file changes: %s", err.message)
+        except GLib.Error:
+            logging.exception("Could not monitor file changes")
             self.window.send_notification(_("Could not save Safe"))
         else:
             if conflicts:
                 dialog = SavingConflictDialog(self.window, dbm, on_save_callback)
-                dialog.present()
+                dialog.present(self.window)
             else:
                 self.database_manager.save_async(on_save_callback)
 
@@ -565,7 +545,8 @@ class UnlockedDatabase(Gtk.Box):
         timeout = gsecrets.config_manager.get_database_lock_timeout() * 60
         if timeout:
             self._lock_timer_handler = GLib.timeout_add_seconds(
-                timeout, self.lock_timeout_database
+                timeout,
+                self.lock_timeout_database,
             )
 
     def start_save_loop(self):
@@ -573,26 +554,26 @@ class UnlockedDatabase(Gtk.Box):
         self.save_loop = GLib.timeout_add_seconds(30, self.threaded_save_loop)
 
     def threaded_save_loop(self) -> bool:
-        """Saves the safe as long as it returns True."""
+        """Return True if the safe was saved."""
         if gsecrets.config_manager.get_save_automatically():
             self.auto_save_database()
 
         return GLib.SOURCE_CONTINUE
 
     def go_back(self):
-        if self.props.selection_mode:
-            self.props.selection_mode = False
-            self.props.mode = self.Mode.GROUP
-            return
-        if self.props.search_active:
+        if self._search_active:
             self.props.search_active = False
-            self.props.mode = self.Mode.GROUP
-            return
-        if self.props.current_element.is_root_group:
             return
 
-        parent = self.props.current_element.parentgroup
-        self.show_browser_page(parent)
+        if self._selection_mode:
+            self.props.selection_mode = False
+            return
+
+        self._navigate_back()
+
+    @Gtk.Template.Callback()
+    def _on_selection_go_back_button_clicked(self, _button):
+        self.headerbar.on_go_back_button_clicked(None)
 
     @GObject.Property(type=bool, default=False)
     def search_active(self) -> bool:
@@ -607,39 +588,131 @@ class UnlockedDatabase(Gtk.Box):
 
     @search_active.setter  # type: ignore
     def search_active(self, value: bool) -> None:
-        """Set the search mode
+        """Set the search mode.
 
         :param value: new search_active
         """
-        if (
-            self.database_manager.props.locked
-            or self.selection_mode
-            or self.in_edit_page
-        ):
-            return
-
         self._search_active = value
-        if self._search_active:
-            self.props.mode = self.Mode.SEARCH
-            self._unlocked_db_stack.set_visible_child_name("search")
-        else:
-            self.show_browser_page(self.current_element)
+
+        if not value:
+            self.browsing_panel.set_search(None)
 
     # TODO This property does not emit a notify signal when the manager locked
     # property is updated.
     @GObject.Property(type=bool, default=False)
     def database_locked(self):
-        """Get database lock status
+        """Get database lock status.
 
         :returns: True if the database is locked
         :rtype: bool
         """
         return self.database_manager.props.locked
 
-    @GObject.Property(type=int, default=0)
-    def mode(self) -> int:
-        return self._mode
+    @GObject.Property(type=bool, default=False)
+    def selection_mode(self) -> bool:
+        return self._selection_mode
 
-    @mode.setter  # type: ignore
-    def mode(self, new_mode: int) -> None:
-        self._mode = new_mode
+    @selection_mode.setter  # type: ignore
+    def selection_mode(self, value: bool) -> None:
+        self._selection_mode = value
+        self._selection_mode_action_bar.set_revealed(value)
+
+        if value:
+            self.headerbar_stack.props.visible_child = self._selection_mode_headerbar
+        else:
+            self.headerbar_stack.props.visible_child = self.headerbar
+            self._clear_selection()
+
+    @Gtk.Template.Callback()
+    def _on_selection_cancel_clicked(self, _button):
+        self.props.selection_mode = False
+
+    @Gtk.Template.Callback()
+    def _on_clear_selection_clicked(self, _button):
+        self.start_database_lock_timer()
+        self._clear_selection()
+
+    @Gtk.Template.Callback()
+    def _on_cut_selection_clicked(self, _button):
+        self.start_database_lock_timer()
+        self._selection_manager.cut_selection()
+        self._update_selection()
+
+    @Gtk.Template.Callback()
+    def _on_paste_selection_clicked(self, _button):
+        self.start_database_lock_timer()
+        self._selection_manager.paste_selection()
+        self._update_selection()
+
+    @Gtk.Template.Callback()
+    def _on_delete_selection_clicked(self, _button):
+        self.start_database_lock_timer()
+        if (
+            element := self.browsing_panel.selection_model.selected_item
+        ) and element.selected:
+            self._split_view.props.content = self._empty_page
+            self._navigate_back()
+
+        self._selection_manager.delete_selection()
+        self._update_selection()
+
+    def add_selection(self, element):
+        if element.is_group:
+            self._selection_manager.add_group(element)
+        else:
+            self._selection_manager.add_entry(element)
+
+        self._update_selection()
+
+    def remove_selection(self, element):
+        if element.is_group:
+            self._selection_manager.remove_group(element)
+        else:
+            self._selection_manager.remove_entry(element)
+
+        self._update_selection()
+
+    def _clear_selection(self):
+        self._selection_manager.clear_selection()
+        self._update_selection()
+
+    def _update_selection(self):
+        n_selected = self._selection_manager.selected_elements
+        cut_mode = self._selection_manager.cut_mode
+        non_empty_selection = n_selected > 0
+
+        self._cut_selection_button.props.sensitive = non_empty_selection
+        self._delete_selection_button.props.sensitive = non_empty_selection
+        self._clear_selection_button.props.sensitive = non_empty_selection
+
+        if n_selected == 0:
+            title = _("Select Items")
+        else:
+            title = ngettext("{} Selected", "{} Selected", n_selected).format(
+                n_selected,
+            )
+
+        self._selection_mode_title.props.title = title
+
+        self._delete_selection_button.props.sensitive = n_selected > 0
+
+        if cut_mode:
+            visible_child = self._cut_selection_button
+        else:
+            visible_child = self._paste_selection_button
+
+        self._cut_paste_button_stack.props.visible_child = visible_child
+
+    def _update_selection_on_collapsed(self):
+        nav = self._split_view
+        hide_selection = nav.props.collapsed and not nav.props.show_content
+        self.browsing_panel.hide_selection(hide_selection)
+
+    def _on_show_content_notify(self, _split_view, _pspec):
+        self._update_selection_on_collapsed()
+
+    def _on_collapsed_notify(self, _split_view, _pspec):
+        self._update_selection_on_collapsed()
+
+    def _on_search_changed(self, search_entry):
+        self.browsing_panel.set_search(search_entry.props.text)
