@@ -1,19 +1,19 @@
 # SPDX-License-Identifier: GPL-3.0-only
 from __future__ import annotations
 
-from gi.repository import Gtk
+from gi.repository import Adw, Gtk
 
-RESOURCE = "/org/gnome/World/Secrets/images/welcome.png"
+from gsecrets import const
 
 
 @Gtk.Template(resource_path="/org/gnome/World/Secrets/gtk/welcome_page.ui")
-class WelcomePage(Gtk.Box):
+class WelcomePage(Adw.Bin):
     __gtype_name__ = "WelcomePage"
 
-    _app_logo = Gtk.Template.Child()
+    _status_page = Gtk.Template.Child()
 
     def __init__(self):
-        """Welcome Page widget"""
+        """Welcome Page widget."""
         super().__init__()
 
-        self._app_logo.set_from_resource(RESOURCE)
+        self._status_page.set_icon_name(const.APP_ID)
