@@ -10,6 +10,7 @@ from gi.repository import Adw, Gio, GLib, GObject, Gtk
 import gsecrets.config_manager
 from gsecrets.create_database import CreateDatabase
 from gsecrets.database_manager import DatabaseManager
+from gsecrets.provider.providers import Providers
 from gsecrets.save_dialog import SaveDialog
 from gsecrets.settings_dialog import SettingsDialog
 from gsecrets.unlock_database import UnlockDatabase
@@ -44,6 +45,15 @@ class Window(Adw.ApplicationWindow):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
 
+        self.key_providers = Providers(self)
+
+        for provider in self.key_providers.get_key_providers():
+            if not provider.available:
+                continue
+
+            provider.connect(provider.show_message, self._on_show_message)
+            provider.connect(provider.hide_message, self._on_hide_message)
+
         self.application = self.get_application()
 
         self.assemble_window()
@@ -56,27 +66,41 @@ class Window(Adw.ApplicationWindow):
         toast = Adw.Toast.new(notification)
         self.toast_overlay.add_toast(toast)
 
+    def _on_show_message(self, _provider, label):
+        self.show_banner(label)
+
+    def _on_hide_message(self, _provider):
+        self.close_banner()
+
+    def show_banner(self, label: str) -> None:
+        if self.view == self.View.UNLOCK_DATABASE:
+            self._unlock_database_bin.props.child.show_banner(label)
+        elif self.view == self.View.CREATE_DATABASE and (
+            create_view := self._create_view
+        ):
+            create_view.show_banner(label)
+
+    def close_banner(self):
+        if self.view == self.View.UNLOCK_DATABASE:
+            self._unlock_database_bin.props.child.close_banner()
+        elif self.view == self.View.CREATE_DATABASE and (
+            create_view := self._create_view
+        ):
+            create_view.close_banner()
+
+    def close_notification(self, toast: Adw.Toast) -> None:
+        toast.dismiss()
+
     def assemble_window(self) -> None:
         window_size = gsecrets.config_manager.get_window_size()
         self.set_default_size(window_size[0], window_size[1])
 
-        self.apply_theme()
-
         if self.props.application.development_mode:
             self.add_css_class("devel")
 
-    def apply_theme(self) -> None:
-        """Set the bright/dark theme depending on the configuration"""
-        manager = Adw.StyleManager.get_default()
-        if not manager.props.system_supports_color_schemes:
-            dark_theme = gsecrets.config_manager.get_dark_theme()
-            if dark_theme:
-                manager.props.color_scheme = Adw.ColorScheme.PREFER_DARK
-            else:
-                manager.props.color_scheme = Adw.ColorScheme.DEFAULT
-
     def do_enable_debugging(  # pylint: disable=arguments-differ
-        self, toggle: bool
+        self,
+        toggle: bool,
     ) -> bool:
         if not self.application.development_mode:
             logging.warning("The inspector is not enabled in non-development builds")
@@ -84,20 +108,8 @@ class Window(Adw.ApplicationWindow):
 
         return Adw.ApplicationWindow.do_enable_debugging(self, toggle)
 
-    def do_size_allocate(
-        self, width: int, height: int, baseline: int
-    ) -> None:  # pylint: disable=arguments-differ
-        # pylint: disable=arguments-differ
-        """Handler for resizing event. It is used to check if
-        the layout needs to be updated."""
-        new_mobile_layout = width < 700
-        if new_mobile_layout != self.props.mobile_layout:
-            self.props.mobile_layout = new_mobile_layout
-
-        Adw.ApplicationWindow.do_size_allocate(self, width, height, baseline)
-
     def invoke_initial_screen(self) -> None:
-        """Present the first start screen if required or autoload files
+        """Present the first start screen if required or autoload files.
 
         If the configuration is set to automatically load the last
         opened safe, this function does that. If it is not set to
@@ -107,20 +119,15 @@ class Window(Adw.ApplicationWindow):
         """
         if gsecrets.config_manager.get_first_start_screen():
             # simply load the last opened file
-            filepath = None
-            gfile: Gio.File = Gio.File.new_for_uri(
-                gsecrets.config_manager.get_last_opened_database()
-            )
-            if gfile.query_exists():
+            uri = gsecrets.config_manager.get_last_opened_database()
+            gfile: Gio.File = Gio.File.new_for_uri(uri)
+            recents = Gtk.RecentManager.get_default()
+            if gfile.query_exists() and recents.has_item(uri):
                 filepath = gfile.get_path()
                 logging.debug("Opening last opened database: %s", filepath)
                 self.start_database_opening_routine(filepath)
                 return
 
-        # Display the screen with last opened files (or welcome page)
-        if not gsecrets.config_manager.get_last_opened_list():
-            logging.debug("No recent files saved")
-
         self.view = self.View.WELCOME
 
     #
@@ -134,54 +141,39 @@ class Window(Adw.ApplicationWindow):
             self._open_database_in_window(path)
             return
 
-        opening_dialog = Gtk.FileChooserNative.new(
-            # NOTE: Filechooser title for opening an existing keepass safe kdbx file
-            _("Select Safe"),
-            self,
-            Gtk.FileChooserAction.OPEN,
-            None,
-            None,
-        )
-
         keepass_filter = Gtk.FileFilter()
         keepass_filter.add_mime_type("application/x-keepass2")
         keepass_filter.add_pattern("*.kdbx")
-        # NOTE: KeePass + version number is a proper name, do not translate
-        keepass_filter.set_name(_("KeePass 3.1/4 Database"))
-        opening_dialog.add_filter(keepass_filter)
+        # TRANSLATORS Safe as in strongbox.
+        keepass_filter.set_name("KeePass 3.1/4 " + _("Safe"))
 
         binary_filter = Gtk.FileFilter()
         binary_filter.add_mime_type("application/octet-stream")
         binary_filter.set_name(_("Any file type"))
-        opening_dialog.add_filter(binary_filter)
 
-        opening_dialog.connect(
-            "response",
-            self._on_open_filechooser_response,
-            opening_dialog,
-        )
-        opening_dialog.show()
+        filters = Gio.ListStore.new(Gtk.FileFilter)
+        filters.append(keepass_filter)
+        filters.append(binary_filter)
 
-    def _on_open_filechooser_response(
-        self,
-        dialog: Gtk.Dialog,
-        response: Gtk.ResponseType,
-        _dialog: Gtk.Dialog,
-    ) -> None:
-        dialog.destroy()
-        if response == Gtk.ResponseType.ACCEPT:
-            db_gfile = dialog.get_file()
-            if db_gfile is None:
-                logging.debug("No file selected")
-                return
+        dialog = Gtk.FileDialog.new()
+        dialog.props.title = _("Select Safe")
+        dialog.props.filters = filters
 
-            db_filename = db_gfile.get_path()
-            logging.debug("File selected: %s", db_filename)
+        dialog.open(self, None, self._on_open_filechooser_response)
 
-            self._open_database_in_window(db_filename)
-
-        elif response == Gtk.ResponseType.CANCEL:
-            logging.debug("File selection canceled")
+    def _on_open_filechooser_response(self, dialog, result):
+        try:
+            db_gfile = dialog.open_finish(result)
+        except GLib.Error as err:
+            if not err.matches(Gtk.DialogError.quark(), Gtk.DialogError.DISMISSED):
+                logging.exception("Could not open file")
+        else:
+            if db_filename := db_gfile.get_path():
+                logging.debug("File selected: %s", db_filename)
+                self._open_database_in_window(db_filename)
+            else:
+                logging.error("File does not have a path, is in a virtual file system?")
+                self.send_notification(_("File does not have a valid path"))
 
     def _open_database_in_window(self, filepath):
         if self.unlocked_db:
@@ -204,24 +196,23 @@ class Window(Adw.ApplicationWindow):
                     if auto_save:
                         self.unlocked_db.auto_save_database()
 
-                    self.unlocked_db.do_dispose()
+                    self.unlocked_db.inner_dispose()
                     self.unlocked_db = None
                     self.start_database_opening_routine(filepath)
 
+            elif not is_current:
+                app = self.props.application
+                window = app.new_window()
+                window.start_database_opening_routine(filepath)
+                window.present()
             else:
-                if not is_current:
-                    app = self.props.application
-                    window = app.new_window()
-                    window.start_database_opening_routine(filepath)
-                    window.present()
-                else:
-                    self.send_notification(_("Safe is already open"))
+                self.send_notification(_("Safe is already open"))
 
         else:
             self.start_database_opening_routine(filepath)
 
     def start_database_opening_routine(self, filepath: str) -> None:
-        """Start opening a safe file"""
+        """Start opening a safe file."""
         # TODO Use a Gio.File instead of a path.
         database = Gio.File.new_for_path(filepath)
         unlock_db = UnlockDatabase(self, database)
@@ -235,38 +226,31 @@ class Window(Adw.ApplicationWindow):
 
     def on_new_database_action(self, _action: Gio.Action, _param: GLib.Variant) -> None:
         """Callback function to create a new safe."""
-        creation_dialog = Gtk.FileChooserNative.new(
-            # NOTE: Filechooser title for creating a new keepass safe kdbx file
-            _("Create Safe"),
-            self,
-            Gtk.FileChooserAction.SAVE,
-            _("_Create"),
-            None,
-        )
-        creation_dialog.set_current_name(_("Safe") + ".kdbx")
-        creation_dialog.set_modal(True)
-
         filter_text = Gtk.FileFilter()
-        # NOTE: KeePass + version number is a proper name, do not translate
-        filter_text.set_name(_("KeePass 3.1/4 Database"))
+        # TRANSLATORS Safe as in strongbox.
+        filter_text.set_name("KeePass 3.1/4 " + _("Safe"))
         filter_text.add_mime_type("application/x-keepass2")
-        creation_dialog.add_filter(filter_text)
 
-        creation_dialog.connect(
-            "response", self._on_create_filechooser_response, creation_dialog
-        )
-        creation_dialog.show()
+        filters = Gio.ListStore.new(Gtk.FileFilter)
+        filters.append(filter_text)
 
-    def _on_create_filechooser_response(
-        self, dialog: Gtk.Dialog, response: Gtk.ResponseType, _dialog: Gtk.Dialog
-    ) -> None:
-        dialog.destroy()
-        if response == Gtk.ResponseType.ACCEPT:
-            gfile = dialog.get_file()
-            if gfile is None:
-                logging.debug("No file selected")
-                return
+        dialog = Gtk.FileDialog.new()
+        # NOTE: Filechooser title for creating a new keepass safe kdbx file
+        dialog.props.title = _("Create Safe")
+        dialog.props.filters = filters
+        dialog.props.accept_label = _("_Create")
+        # NOTE: Safe as in strongbox
+        dialog.props.initial_name = _("Safe") + ".kdbx"
 
+        dialog.save(self, None, self._on_create_filechooser_response)
+
+    def _on_create_filechooser_response(self, dialog, result):
+        try:
+            gfile = dialog.save_finish(result)
+        except GLib.Error as err:
+            if not err.matches(Gtk.DialogError.quark(), Gtk.DialogError.DISMISSED):
+                logging.exception("Could not save file")
+        else:
             filepath = gfile.get_path()
             if self.unlocked_db:
                 auto_save = gsecrets.config_manager.get_save_automatically()
@@ -276,7 +260,7 @@ class Window(Adw.ApplicationWindow):
 
                 if is_open:
                     self.send_notification(
-                        _("Cannot create Safe: Safe is already open")
+                        _("Cannot create Safe: Safe is already open"),
                     )
                     return
 
@@ -290,7 +274,7 @@ class Window(Adw.ApplicationWindow):
                         if auto_save:
                             self.unlocked_db.auto_save_database()
 
-                        self.unlocked_db.do_dispose()
+                        self.unlocked_db.inner_dispose()
                         self.unlocked_db = None
                         self.create_new_database(filepath)
                 else:
@@ -308,15 +292,15 @@ class Window(Adw.ApplicationWindow):
         self._main_view.set_visible_child_name("spinner")
 
         stock_db_file: Gio.File = Gio.File.new_for_uri(
-            "resource:///org/gnome/World/Secrets/database.kdbx"
+            "resource:///org/gnome/World/Secrets/database.kdbx",
         )
         new_db_file: Gio.File = Gio.File.new_for_path(filepath)
 
         def unlock_callback(database_manager, result):
             try:
                 database_manager.unlock_finish(result)
-            except GLib.Error as err:
-                logging.debug("Could not unlock safe: %s", err.message)
+            except GLib.Error:
+                logging.exception("Could not unlock safe")
                 self.invoke_initial_screen()
                 self.send_notification(_("Could not create new Safe"))
             else:
@@ -330,27 +314,37 @@ class Window(Adw.ApplicationWindow):
         def copy_callback(gfile, result):
             try:
                 gfile.copy_finish(result)
-            except GLib.Error as err:
-                logging.debug("Could not copy new database: %s", err.message)
+            except GLib.Error:
+                logging.exception("Could not copy new database")
                 self.invoke_initial_screen()
                 self.send_notification(_("Could not create new Safe"))
                 self._spinner.stop()
             else:
-                database_manager = DatabaseManager(filepath)
+                database_manager = DatabaseManager(self.key_providers, filepath)
                 database_manager.unlock_async(
                     "liufhre86ewoiwejmrcu8owe",
                     callback=unlock_callback,
                 )
 
-        stock_db_file.copy_async(
-            new_db_file,
-            Gio.FileCopyFlags.OVERWRITE,
-            GLib.PRIORITY_DEFAULT,
-            None,
-            None,
-            None,
-            copy_callback,
-        )
+        if GLib.check_version(2, 81, 0) is None:
+            stock_db_file.copy_async(
+                new_db_file,
+                Gio.FileCopyFlags.OVERWRITE,
+                GLib.PRIORITY_DEFAULT,
+                None,
+                None,
+                copy_callback,
+            )
+        else:
+            stock_db_file.copy_async(
+                new_db_file,
+                Gio.FileCopyFlags.OVERWRITE,
+                GLib.PRIORITY_DEFAULT,
+                None,
+                None,
+                None,
+                copy_callback,
+            )
 
     def save_window_size(self):
         width = self.get_width()
@@ -361,14 +355,15 @@ class Window(Adw.ApplicationWindow):
     def force_close(self):
         """Method to close a window with unsaved changes.
 
-        Should only be called from SaveDialog and QuitConflictDialog."""
+        Should only be called from SaveDialog and QuitConflictDialog.
+        """
         if unlocked_db := self.unlocked_db:
             unlocked_db.database_manager.is_dirty = False
 
         self.close()
 
     def do_close_request(self) -> bool:  # pylint: disable=arguments-differ
-        """This is invoked when the window close button is pressed.
+        """Invoked when the window close button is pressed.
 
         Only the app.quit action is called. This action cleans up stuff
         and will invoke the on_application_shutdown() method.
@@ -385,32 +380,31 @@ class Window(Adw.ApplicationWindow):
         def on_save(dbm, result):
             try:
                 is_saved = dbm.save_finish(result)
-            except GLib.Error as err:
-                logging.error("Could not save Safe: %s", err.message)
-                self.send_notification(_("Could not save Safe"))
+            except GLib.Error:
+                logging.exception("Could not save Safe")
+                self.show_quit_confirmation_dialog()
             else:
                 if is_saved:
                     self.close()
                 else:
                     # This shouldn't happen
-                    self.send_notification(_("Could not save Safe"))
+                    self.show_quit_confirmation_dialog()
 
         def on_check_file_changes(dbm, result):
             try:
                 conflicts = dbm.check_file_changes_finish(result)
-            except GLib.Error as err:
-                logging.error("Could not monitor file changes: %s", err.message)
+            except GLib.Error:
+                logging.exception("Could not monitor file changes")
                 self.send_notification(_("Could not save safe"))
             else:
                 if conflicts:
                     dialog = QuitConflictDialog(self, dbm, on_save)
                     dialog.present()
+                elif gsecrets.config_manager.get_save_automatically():
+                    self.unlocked_db.database_manager.save_async(on_save)
                 else:
-                    if gsecrets.config_manager.get_save_automatically():
-                        self.unlocked_db.database_manager.save_async(on_save)
-                    else:
-                        save_dialog = SaveDialog(self)
-                        save_dialog.present()
+                    save_dialog = SaveDialog(self)
+                    save_dialog.present(self)
 
         if is_database_dirty:
             dbm.check_file_changes_async(on_check_file_changes)
@@ -419,6 +413,19 @@ class Window(Adw.ApplicationWindow):
         self.save_window_size()
         return False
 
+    def show_quit_confirmation_dialog(self):
+        def on_discard(self, _dialog, _response):
+            self.force_close()
+
+        # TODO Set the body of the message based on the error kind.
+        dialog = Adw.AlertDialog.new(_("Could not save Safe"), None)
+        dialog.add_response("discard", _("_Quit Without Saving"))
+        dialog.add_response("cancel", _("_Don't Quit"))
+        dialog.set_response_appearance("discard", Adw.ResponseAppearance.DESTRUCTIVE)
+        dialog.set_default_response("cancel")
+        dialog.connect("response::discard", on_discard)
+        dialog.present(self)
+
     def do_unrealize(self):  # pylint: disable=arguments-differ
         if self.unlocked_db:
             self.unlocked_db.cleanup()
@@ -445,12 +452,19 @@ class Window(Adw.ApplicationWindow):
         self.add_action(new_database_action)
         new_database_action.connect("activate", self.on_new_database_action)
 
+        def on_visible_dialog_notify(window, _pspec):
+            enabled = window.props.visible_dialog is None
+            go_back_action.set_enabled(enabled)
+
         go_back_action = Gio.SimpleAction.new("go_back", None)
         self.add_action(go_back_action)
         go_back_action.connect("activate", self.on_go_back_action)
+        # NOTE We need to do this to avoid consuming Esc events
+        self.connect("notify::visible-dialog", on_visible_dialog_notify)
 
         open_database_action = Gio.SimpleAction.new(
-            "open_database", GLib.VariantType("s")
+            "open_database",
+            GLib.VariantType("s"),
         )
         self.add_action(open_database_action)
         open_database_action.connect("activate", self.on_open_database_action)
@@ -463,6 +477,7 @@ class Window(Adw.ApplicationWindow):
             "db.add_group",
             "db.settings",
             "db.undo_delete",
+            "db.undo_attribute_delete",
             "element.delete",
             "entry.duplicate",
             "entry.references",
@@ -510,42 +525,39 @@ class Window(Adw.ApplicationWindow):
             action_db.selection_mode_headerbar.on_selection_action(param)
         elif name == "db.undo_delete":
             action_db.undo_delete()
+        elif name == "db.undo_attribute_delete":
+            action_db.undo_attribute_delete()
         elif name in ["db.save", "db.save_dirty"]:
             action_db.save_database()
         elif name in ["db.add_entry", "db.add_group"]:
-            if (
-                action_db.props.database_locked
-                or action_db.props.selection_mode
-                or action_db.in_edit_page
-                or action_db.props.search_active
-            ):
+            if action_db.props.database_locked or action_db.props.selection_mode:
                 return
 
+            if action_db.props.search_active:
+                action_db.props.search_active = False
+
             if name == "db.add_entry":
                 action_db.on_add_entry_action()
             else:
                 action_db.on_add_group_action()
 
     def on_about_action(self, _action: Gio.Action, _param: GLib.Variant) -> None:
-        """Invoked when we click "about" in the main menu"""
+        """Invoked when we click "about" in the main menu."""
         builder = Gtk.Builder.new_from_resource(
-            "/org/gnome/World/Secrets/about_dialog.ui"
+            "/org/gnome/World/Secrets/about_dialog.ui",
         )
         about_dialog = builder.get_object("about_dialog")
-        about_dialog.set_transient_for(self)
-        about_dialog.present()
+        about_dialog.present(self)
 
     def on_settings_action(self, _action: Gio.Action, _param: GLib.Variant) -> None:
-        SettingsDialog(self).present()
+        SettingsDialog().present(self)
 
     def on_go_back_action(self, _action: Gio.Action, _param: GLib.Variant) -> None:
         if self.view == self.View.CREATE_DATABASE:
             if create_view := self._create_view:
                 create_view.go_back()
-        else:
-            if action_db := self.unlocked_db:
-                if not action_db.props.database_locked:
-                    action_db.go_back()
+        elif (action_db := self.unlocked_db) and not action_db.props.database_locked:
+            action_db.go_back()
 
     @Gtk.Template.Callback()
     def on_back_button_pressed(
